# -*- coding: utf-8 -*-
# Copyright 2009-2014, Red Hat, Inc.
# License: GPL-2.0+ <http://spdx.org/licenses/GPL-2.0+>
# See the LICENSE file for more details on Licensing

from __future__ import absolute_import

import os
import subprocess
import pipes

from libtaskotron.directives import BaseDirective
from libtaskotron.exceptions import TaskotronDirectiveError
from libtaskotron.logger import log

DOCUMENTATION = """
module: bash_directive
short_description: execute a bash command
description: |
  Execute a bash command. You can also execute a bash script using ``sh file`` as command.
  The code is run in subprocess.
parameters:
  command:
    required: true
    description: bash command to be executed
    type: str
returns: |
  The stdout of executed bash command.


  If this command is supposed to be the main check of the task, it is supposed to provide text
  output in a :ref:`resultyaml-format`. This format is then used in the
  :ref:`resultsdb directive <resultsdb_directive>`.

raises: |
  * :class:`.TaskotronDirectiveError`: if ``command`` was not provided or executed command returned
    non-zero returncode.
version_added: 0.4.12
"""

EXAMPLES = """

This executes the shell command in subprocess::

  - name: pull docker image
    bash:
        command: docker pull httpd

You can also run bash script::

  - name: execute bash script
    bash:
        command: sh test.sh
    export: test_output

Bash directive can work as file loader::

  - name: load file to formula variable
    bash:
        command: cat some_file
    export: loaded_file

"""

directive_class = 'BashDirective'


class BashDirective(BaseDirective):
    def process(self, params, arg_data):
        basedir = os.path.dirname(os.path.abspath(arg_data['task']))

        try:
            cmd = [pipes.quote(c) for c in params['command'].split()]
        except KeyError as e:
            raise TaskotronDirectiveError('No command provided.')

        log.info("Executing Bash: %s", cmd)

        try:
            output = subprocess.check_output(cmd, stderr=subprocess.STDOUT, cwd=basedir)
        except subprocess.CalledProcessError as e:
            raise TaskotronDirectiveError(e)
        except OSError as e:
            raise TaskotronDirectiveError(e)

        log.debug(output)

        return output
